////////////////////////////////////////////////////////////////////////////////
//  
//  AbstractGraph.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 31/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "AbstractGraph.h"

// Реализуем здесь элементарные методы, а так же не очень элементарные, пусть и с теоретическим оверхедом.

@implementation AbstractGraph

#pragma mark - Operations with vertices

- (NSSet*) vertexSet
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) containsVertex: (vertexType) vertex
{
  return [[self vertexSet] containsObject: vertex];
}

- (NSUInteger) vertexCount
{
  return [[self vertexSet] count];
}

- (NSUInteger) degreeOfVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return 0;
}

- (BOOL) isOddVertex: (vertexType) vertex
{
  return (BOOL)([self degreeOfVertex: vertex] % 2);
}

- (BOOL) isEvenVertex: (vertexType) vertex
{
  return ![self isOddVertex: vertex];
}

- (BOOL) isIsolatedVertex: (vertexType) vertex
{
  return [self degreeOfVertex: vertex] == 0;
}

- (BOOL) isPendantVertex: (vertexType) vertex
{
  return [self degreeOfVertex: vertex] == 1;
}

- (NSSet*) openNeighbourhoodSetOf: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSSet*) closedNeighbourhoodSetOf: (vertexType) vertex
{
  return [[self openNeighbourhoodSetOf: vertex] setByAddingObject: vertex];
}

- (BOOL) isVertex: (vertexType) vertex1 neighbourOf: (vertexType) vertex2
{
  return [[self openNeighbourhoodSetOf: vertex2] containsObject: vertex1];
}

- (NSUInteger) neighboursCountOfVertex: (vertexType) vertex
{
  return [[self openNeighbourhoodSetOf: vertex] count];
}

#pragma mark - Operations with vertices & edges

- (vertexType) oppositeVertexOfEdge: (edgeType) edge currentVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

#pragma mark - Operations with edges

- (NSSet*) edgeSet
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) containsEdge: (edgeType) edge
{
  return [[self edgeSet] containsObject: edge];
}

- (NSUInteger) edgeCount
{
  return [[self edgeSet] count];
}

- (BOOL) isLoopEdge: (edgeType) edge
{
  [self subclassResponsibility: _cmd];
  
  return NO;
}

#pragma mark - Universal method for graphs traversal

- (NSEnumerator*) possibleWaysOutEnumeratorOf: (vertexType) vertex;
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

#pragma mark - Testing for trivial graph types

- (BOOL) isNullGraph
{
  return [self vertexCount] == 0;
}

- (BOOL) isSingletonGraph
{
  return [self vertexCount] == 1;
}

- (BOOL) isEdgelessGraph
{
  return [self edgeCount] == 0;
}

@end
