////////////////////////////////////////////////////////////////////////////////
//  
//  AbstractUndirectedGraph.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 05/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "AbstractUndirectedGraph.h"

@implementation AbstractUndirectedGraph

#pragma mark - Graph protocol overrides

- (NSUInteger) degreeOfVertex: (vertexType) vertex
{
  // THINK: What about self-loops?
  return [self neighboursCountOfVertex: vertex];
}

- (NSEnumerator*) possibleWaysOutEnumeratorOf: (vertexType) vertex
{
  return [[self incidentEdgesSetOf: vertex] objectEnumerator];
}

- (vertexType) oppositeVertexOfEdge: (edgeType) edge currentVertex: (vertexType) vertex
{
  Pair* incidentVertices = [self incidentVerticesPairOf: edge];
  
  vertexType u = incidentVertices.object1;
  
  vertexType v = incidentVertices.object2;
  
  if([vertex isEqual: u])
  {
    return v;
  }
  else if([vertex isEqual: v])
  {
    return u;
  }
  
  // TODO: raise exception "Invalid Argument".
  
  return nil;
}

#pragma mark - Operations with vertices & edges

- (edgeType) edgeBetween: (vertexType) u and: (vertexType) v
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSSet*) allEdgesBetween: (vertexType) u and: (vertexType) v
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) containsEdgeBetween: (vertexType) u and: (vertexType) v
{
  return [self edgeBetween: u and: v] != nil;
}

- (Pair*) incidentVerticesPairOf: (edgeType) edge
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSSet*) incidentEdgesSetOf: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) isVertex: (vertexType) vertex incidentToEdge: (edgeType) edge
{
  return [[self incidentEdgesSetOf: vertex] containsObject: edge];
}

- (BOOL) isEdge: (edgeType) edge incidentToVertex: (vertexType) vertex
{
  return [self isVertex: vertex incidentToEdge: edge];
}

#pragma mark - Operations with edges

- (NSSet*) adjacentEdgesSetOf: (edgeType) edge
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) isEdge: (edgeType) edge1 adjacentTo: (edgeType) edge2
{
  return [[self adjacentEdgesSetOf: edge1] containsObject: edge2];
}

@end
