////////////////////////////////////////////////////////////////////////////////
//  
//  ConnectedComponentClusterer.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 30/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "ConnectedComponentClusterer.h"

#import "UndirectedGraph.h"

#import "GraphTraversal.h"

@implementation ConnectedComponentClusterer

+ (ConnectedComponentClusterer*) clusterer
{
  return [[[self class] new] autorelease];
}

- (NSSet*) clusterGraph: (id<UndirectedGraph>) graph
{
  NSMutableSet* components = [NSMutableSet set];
  
  currentComponent = [[NSMutableSet alloc] init];
  
  finishedVertices = [[NSMutableSet alloc] initWithCapacity: [graph vertexCount]];
  
  BreadthFirstTraversal* traversal = [BreadthFirstTraversal traversal];
  
  for(vertexType vertex in [graph vertexSet])
  {
    if([finishedVertices containsObject: vertex]) continue;
    
    [traversal traverseGraph: graph startVertex: vertex visitors: self, nil];
    
    [components addObject: currentComponent];
  }
  
  [currentComponent release], currentComponent = nil;
  
  [finishedVertices release], finishedVertices = nil;
  
  return components;
}

#pragma mark - TraversalVisitor

- (void) didVisitVertex:(vertexType) vertex
{
  [currentComponent addObject: vertex];
  
  [finishedVertices addObject: vertex];
}

@end
