////////////////////////////////////////////////////////////////////////////////
//  
//  CyclesDetector.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 20/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "CyclesDetector.h"

#import "EdgeClassifier.h"

#import "GraphTraversal.h"

@implementation CyclesDetector

@synthesize graph;

+ (BOOL) isAcyclicGraph: (id<Graph>) G
{
  return [[[self class] cyclesDetectorWithGraph: G] isAcyclic];
}

+ (NSSet*) cycleEdgesOfGraph: (id<Graph>) G
{
  return [[[self class] cyclesDetectorWithGraph: G] cycleEdges];
}

+ (CyclesDetector*) cyclesDetectorWithGraph: (id<Graph>) G
{
  return [[[[self class] alloc] initWithGraph: G] autorelease];
}

- (id) initWithGraph: (id<Graph>) G
{
  self = [super init];
  
  if(!self) return nil;
  
  edgeClassifier = [[BFTEdgeClassifier alloc] initWithDelegate: self];
  
  cycleEdges = [[NSMutableSet alloc] init];
  
  return self;
}

- (void) dealloc
{
  [edgeClassifier release], edgeClassifier = nil;
  
  [cycleEdges release], cycleEdges = nil;
  
  [super dealloc];
}

- (BOOL) isAcyclic
{
  return [[self cycleEdges] count];
}

- (NSSet*) cycleEdges
{
  BreadthFirstTraversal* traversal = [BreadthFirstTraversal traversal];
  
  [traversal traverseGraph: graph visitors: edgeClassifier, nil];
  
  return [[cycleEdges copy] autorelease];
}

#pragma mark - EdgeClassifierDelegate

- (void) edgeClassifiedAsBackEdge: (edgeType) e
{
  [cycleEdges addObject: e];
}

@end
