////////////////////////////////////////////////////////////////////////////////
//  
//  DirectedSparsePseudographTests.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 06/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "DirectedSparsePseudographTests.h"

#import "DirectedSparsePseudograph.h"

@implementation DirectedSparsePseudographTests

- (void) setUp
{
  [super setUp];
  
  directedPseudograph = [[MutableDirectedSparsePseudograph alloc] init];
  
  NSArray* vertices = [NSArray arrayWithObjects: @"v0", @"v1", @"v2", @"v3", @"v4", @"lonelyV", @"loopV", @"vA", @"vB", nil];
  
  // Добавить все вершины в граф по одной.
  for(NSString* vertex in vertices) [directedPseudograph addVertex: vertex];
  
  // Ребра «звезды».
  [directedPseudograph addEdge: @"e0" from: @"v0" to: @"v1"];
  
  [directedPseudograph addEdge: @"e1" from: @"v1" to: @"v2"];
  
  [directedPseudograph addEdge: @"e2" from: @"v2" to: @"v3"];
  
  [directedPseudograph addEdge: @"e3" from: @"v3" to: @"v4"];
  
  [directedPseudograph addEdge: @"e4" from: @"v4" to: @"v0"];
  
  // Петля.
  [directedPseudograph addEdge: @"loopE" from: @"loopV" to: @"loopV"];
  
  // Кратные ребра.
  [directedPseudograph addEdge: @"eAB0" from: @"vA" to: @"vB"];
  
  [directedPseudograph addEdge: @"eAB1" from: @"vA" to: @"vB"];
  
  [directedPseudograph addEdge: @"eBA" from: @"vB" to: @"vA"];
}

- (void) tearDown
{
  [directedPseudograph release];
  
  [super tearDown];
}

- (void) testContainsVertex
{
  STAssertTrue([directedPseudograph containsVertex: @"v2"], @"Invalid -containsVertex: result.");
  
  STAssertFalse([directedPseudograph containsVertex: @"foreignV"], @"Invalid -containsVertex: result.");
}

- (void) testContainsEdge
{
  STAssertTrue([directedPseudograph containsEdge: @"loopE"], @"Invalid -containsEdge: result.");
  
  STAssertFalse([directedPseudograph containsVertex: @"foreignE"], @"Invalid -containsEdge: result.");
}

- (void) testDegree
{
  NSArray* vertices = [NSArray arrayWithObjects: @"v0", @"v1", @"v2", @"v3", @"v4", nil];
  
  // Все вершины «звезды» имеют одинаковые значения [*]degree.
  for(NSString* vertex in vertices)
  {
    STAssertEquals([directedPseudograph indegreeOfVertex: vertex], (NSUInteger)1, @"Invalid vertex indegree.");
    
    STAssertEquals([directedPseudograph outdegreeOfVertex: vertex], (NSUInteger)1, @"Invalid vertex outdegree.");
    
    STAssertEquals([directedPseudograph degreeOfVertex: vertex], (NSUInteger)2, @"Invalid vertex degree.");
  }
  
  // Одинокая вершина.
  STAssertEquals([directedPseudograph indegreeOfVertex: @"lonelyV"], (NSUInteger)0, @"Invalid vertex indegree.");
  
  STAssertEquals([directedPseudograph outdegreeOfVertex: @"lonelyV"], (NSUInteger)0, @"Invalid vertex outdegree.");
  
  STAssertEquals([directedPseudograph degreeOfVertex: @"lonelyV"], (NSUInteger)0, @"Invalid vertex degree.");
  
  // Петля.
  STAssertEquals([directedPseudograph indegreeOfVertex: @"loopV"], (NSUInteger)1, @"Invalid vertex indegree.");
  
  STAssertEquals([directedPseudograph outdegreeOfVertex: @"loopV"], (NSUInteger)1, @"Invalid vertex outdegree.");
  
  STAssertEquals([directedPseudograph degreeOfVertex: @"loopV"], (NSUInteger)2, @"Invalid vertex degree.");
  
  // Кратные ребра.
  STAssertEquals([directedPseudograph indegreeOfVertex: @"vA"], (NSUInteger)1, @"Invalid vertex indegree.");
  
  STAssertEquals([directedPseudograph outdegreeOfVertex: @"vA"], (NSUInteger)2, @"Invalid vertex outdegree.");
  
  STAssertEquals([directedPseudograph degreeOfVertex: @"vA"], (NSUInteger)3, @"Invalid vertex degree.");
  
  // * * *
  
  STAssertEquals([directedPseudograph indegreeOfVertex: @"vB"], (NSUInteger)2, @"Invalid vertex indegree.");
  
  STAssertEquals([directedPseudograph outdegreeOfVertex: @"vB"], (NSUInteger)1, @"Invalid vertex outdegree.");
  
  STAssertEquals([directedPseudograph degreeOfVertex: @"vB"], (NSUInteger)3, @"Invalid vertex degree.");
}

@end
