////////////////////////////////////////////////////////////////////////////////
//  
//  DistanceRecorder.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 12/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "DistanceRecorder.h"

@implementation DistanceRecorder

+ (DistanceRecorder*) distanceRecorder
{
  return [[[self class] new] autorelease];
}

- (id) init
{
  self = [super init];
  
  if(!self) return nil;
  
  vertexToDistance = [NSMapTable mapTableWithStrongToStrongObjects];
  
  return self;
}

- (void) dealloc
{
  [vertexToDistance release], vertexToDistance = nil;
  
  [super dealloc];
}

- (NSUInteger) distanceFromStartVertexTo: (vertexType) v
{
  NSNumber* d = [vertexToDistance objectForKey: v];
  
  return (d == nil)? NSUIntegerMax : [d unsignedIntegerValue];
}

#pragma mark - TraversalVisitor

- (void) willStartFromVertex: (vertexType) s
{
  // Сбрасываем карту расстояний и текущую вершину перед каждым новым обходом графа.
  [vertexToDistance removeAllObjects];
  
  currentlyVisiting = nil;
  
  [vertexToDistance setObject: [NSNumber numberWithUnsignedInteger: 0] forKey: s];
}

- (void) willVisitVertex: (vertexType) v
{
  currentlyVisiting = v;
}

/*!
 * @exception NSInternalInconsistencyException if for some reason distance of parent could not be found.
 */
- (void) foundUndiscoveredVertex: (vertexType) v
{
  NSNumber* distanceOfParentNumber = [vertexToDistance objectForKey: currentlyVisiting];
  
  if(!distanceOfParentNumber) [NSException raise: NSInternalInconsistencyException format: @"Parent vertex has no associated distance."];
  
  NSUInteger distanceOfParent = [distanceOfParentNumber unsignedIntegerValue];
  
  [vertexToDistance setObject: [NSNumber numberWithUnsignedInteger: distanceOfParent + 1] forKey: v];
}

@end
