////////////////////////////////////////////////////////////////////////////////
//  
//  PredecessorRecorder.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 14/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "PredecessorRecorder.h"

@implementation PredecessorRecorder

+ (PredecessorRecorder*) predecessorRecorder
{
  return [[[self class] new] autorelease];
}

- (id) init
{
  self = [super init];
  
  if(!self) return nil;
  
  vertexToParent = [[NSMapTable mapTableWithStrongToStrongObjects] retain];
  
  return self;
}

- (void) dealloc
{
  [vertexToParent release], vertexToParent = nil;
  
  [super dealloc];
}

- (vertexType) parentOfVertex: (vertexType) v
{
  return [vertexToParent objectForKey: v];
}

#pragma mark - TraversalVisitor

// Сбрасываем карту предков перед каждым новым обходом графа.
- (void) connectedVerticesTraversalDidStart
{
  [vertexToParent removeAllObjects];
  
  currentPredecessor = nil;
}

- (void) willVisitVertex: (vertexType) v
{
  currentPredecessor = v;
}

- (void) foundUndiscoveredVertex: (vertexType) v
{
  // Для стартовой вершины currentPredecessor == 0.
  if(currentPredecessor) [vertexToParent setObject: currentPredecessor forKey: v];
}

- (void) didVisitVertex: (vertexType) v
{
  currentPredecessor = [vertexToParent objectForKey: v];
}

@end
