////////////////////////////////////////////////////////////////////////////////
//  
//  TimeStamper.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 14/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

#import "TraversalVisitor.h"

#import "ElementTypes.h"

/*!
 * An object that being a traversal visitor (both BFT & DFT) records pre- and post-visit times of each vertex in the connected component.
 */
@interface TimeStamper : NSObject <TraversalVisitor>
{
  @private
    NSMapTable* vertexToPrevisitTime;
    
    NSMapTable* vertexToPostvisitTime;
    
    NSUInteger currentTime;
}

/*!
 * Convenience method to create autoreleased TimeStamper object.
 */
+ (TimeStamper*) timeStamper;

/*!
 * Returns pre-visit time of the vertex.
 *
 * @return NSUIntegerMax if vertex v has no associated previsit time. A valid previsit time otherwise.
 */
- (NSUInteger) previsitTimeOfVertex: (vertexType) v;

/*!
 * Returns postvisit time of the vertex.
 *
 * @return NSUIntegerMax if vertex v has no associated postvisit time. A valid postvisit time otherwise.
 */
- (NSUInteger) postvisitTimeOfVertex: (vertexType) v;

@end
