////////////////////////////////////////////////////////////////////////////////
//  
//  TimeStamper.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 14/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "TimeStamper.h"

@implementation TimeStamper

+ (TimeStamper*) timeStamper
{
  return [[[self class] new] autorelease];
}

- (id) init
{
  self = [super init];
  
  if(!self) return nil;
  
  vertexToPrevisitTime = [[NSMapTable mapTableWithStrongToStrongObjects] retain];
  
  vertexToPostvisitTime = [[NSMapTable mapTableWithStrongToStrongObjects] retain];
  
  currentTime = 0;
  
  return self;
}

- (void) dealloc
{
  [vertexToPrevisitTime release], vertexToPrevisitTime = nil;
  
  [vertexToPostvisitTime release], vertexToPostvisitTime = nil;
  
  [super dealloc];
}

- (NSUInteger) previsitTimeOfVertex: (vertexType) v
{
  NSNumber* preTime = [vertexToPrevisitTime objectForKey: v];
  
  return preTime? [preTime unsignedIntegerValue] : NSUIntegerMax;
}

- (NSUInteger) postvisitTimeOfVertex: (vertexType) v
{
  NSNumber* postTime = [vertexToPostvisitTime objectForKey: v];
  
  return postTime? [postTime unsignedIntegerValue] : NSUIntegerMax;
}

#pragma mark - TraversalVisitor

// Сбрасываем времена перед каждым новым обходом графа.
- (void) connectedVerticesTraversalDidStart
{
  [vertexToPrevisitTime removeAllObjects];
  
  [vertexToPostvisitTime removeAllObjects];
  
  currentTime = 0;
}

- (void) willVisitVertex: (vertexType) v
{
  [vertexToPrevisitTime setObject: [NSNumber numberWithUnsignedInteger: currentTime++] forKey: v];
}

- (void) didVisitVertex: (vertexType) v
{
  [vertexToPostvisitTime setObject: [NSNumber numberWithUnsignedInteger: currentTime++] forKey: v];
}

@end
