////////////////////////////////////////////////////////////////////////////////
//  
//  TopologicalSorting.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 19/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "TopologicalSorting.h"

#import "DirectedGraph.h"

#import "TimeStamper.h"

#import "EdgeClassifier.h"

#import "GraphTraversal.h"

@implementation TopologicalSorting

+ (NSArray*) topologicalSortOfDirectedGraph: (id<DirectedGraph>) diGraph
{
  TopologicalSorting* sorting = [[self class] topologicalSortingWithDirectedGraph: diGraph];
  
  return [sorting topologicalSort];
}

+ (TopologicalSorting*) topologicalSortingWithDirectedGraph: (id<DirectedGraph>) diGraph
{
  return [[[[self class] alloc] initWithDirectedGraph: diGraph] autorelease];
}

- (id) initWithDirectedGraph: (id<DirectedGraph>) diGraph
{
  self = [super init];
  
  if(!self) return nil;
  
  directedGraph = [diGraph retain];
  
  timeStamper = [[TimeStamper timeStamper] retain];
  
  edgeClassifier = [[DFTEdgeClassifier alloc] initWithTimeStamper: timeStamper delegate: self];
  
  depthTraversal = [[DepthFirstTraversal traversal] retain];
  
  topologicalOrder = [[NSMutableArray alloc] init];
  
  return self;
}

- (void) dealloc
{
  [directedGraph release], directedGraph = nil;
  
  [timeStamper release], timeStamper = nil;
  
  [edgeClassifier release], edgeClassifier = nil;
  
  [depthTraversal release], depthTraversal = nil;
  
  [topologicalOrder release], topologicalOrder = nil;
  
  [super dealloc];
}

- (NSArray*) topologicalSort
{
  [depthTraversal traverseGraph: directedGraph visitors: timeStamper, edgeClassifier, self, nil];
  
  return topologicalOrder;
}

#pragma mark - EdgeClassifierDelegate

- (void) edgeClassifiedAsBackEdge: (edgeType) e
{
  [NSException raise: NSInvalidArgumentException format: @"Passed directed graph is not acyclic."];
}

#pragma mark - TraversalVisitor

- (void) didVisitVertex: (vertexType) v
{
  // "Push".
  [topologicalOrder insertObject: v atIndex: 0];
}

@end
