////////////////////////////////////////////////////////////////////////////////
//  
//  TopologicalSortingTests.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 19/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "TopologicalSortingTests.h"

#import "DirectedSparsePseudograph.h"

#import "TopologicalSorting.h"

@implementation TopologicalSortingTests

- (void) setUp
{
  directedGraph = [[MutableDirectedSparsePseudograph alloc] init];
}

- (void) tearDown
{
  [directedGraph release], directedGraph = nil;
}

/*!
 * v0 —> v1 —> v2 —> v3 —> v4 —> v5 —> v6 —> v7 —> v8 —> v9
 */
- (void) testTrivialTopologicalSort
{
  NSMutableArray* correctTopologicalOrder = [NSMutableArray array];
  
  vertexType previousVertex = nil;
  
  for(NSUInteger index = 0; index < 10; index++)
  {
    vertexType newVertex = [NSString stringWithFormat: @"v%lu", index];
    
    [directedGraph addVertex: newVertex];
    
    [correctTopologicalOrder addObject: newVertex];
    
    if(previousVertex)
    {
      [directedGraph addEdge: [NSString stringWithFormat: @"e(%lu —> %lu)", index - 1, index] from: previousVertex to: newVertex];
    }
    
    previousVertex = newVertex;
  }
  
  NSArray* topologicalOrder = [TopologicalSorting topologicalSortOfDirectedGraph: directedGraph];
  
  STAssertTrue([topologicalOrder isEqualToArray: correctTopologicalOrder], @"Invalid topological sort.");
}

/*!
 * Testing topological sorting on a clothes graph.
 *
 * Undershirt —> Shirt —> Jacket.
 *
 * Panties —> Jeans —> Boots <— Socks.
 *
 * Watch.
 */
- (void) testClothesSort
{
  // Добавляем вершины.
  NSArray* clothes = [NSArray arrayWithObjects: @"Undershirt", @"Shirt", @"Jacket", @"Panties", @"Jeans", @"Socks", @"Boots", @"Watch", nil];
  
  for(NSString* thing in clothes) [directedGraph addVertex: thing];
  
  // Добавляем ребра.
  [directedGraph addEdge: @"e0" from: @"Undershirt" to: @"Shirt"];
  
  [directedGraph addEdge: @"e1" from: @"Shirt" to: @"Jacket"];
  
  [directedGraph addEdge: @"e2" from: @"Panties" to: @"Jeans"];
  
  [directedGraph addEdge: @"e3" from: @"Jeans" to: @"Boots"];
  
  [directedGraph addEdge: @"e4" from: @"Socks" to: @"Boots"];
  
  // Сортируем граф.
  NSArray* topologicalOrder = [TopologicalSorting topologicalSortOfDirectedGraph: directedGraph];
  
  // Проверяем порядок вершин.
  #define INDEX_OF(a) [topologicalOrder indexOfObject: a]
  
  STAssertTrue(INDEX_OF(@"Undershirt") < INDEX_OF(@"Shirt") < INDEX_OF(@"Jacket"), @"Invalid topological sort.");
  
  STAssertTrue(INDEX_OF(@"Panties") < INDEX_OF(@"Jeans") < INDEX_OF(@"Boots"), @"Invalid topological sort.");
  
  STAssertTrue(INDEX_OF(@"Socks") < INDEX_OF(@"Boots"), @"Invalid topological sort.");
}

@end
