////////////////////////////////////////////////////////////////////////////////
//  
//  TraversalVisitor.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 29/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

#import "ElementTypes.h"

/*!
 * This visitor protocol in used by both BFT & DFT algorithms.
 */
@protocol TraversalVisitor <NSObject>

@optional

/*!
 * Invoked on every vertex before start of the traversal.
 */
- (void) initializeVertex: (vertexType) v;

/*!
 * Invoked on the start vertex before start of the traversal.
 */
- (void) willStartFromVertex: (vertexType) s;

/*!
 * Invoked before traversal of the first connected component if user called whole-graph traversal.
 */
- (void) graphTraversalDidStart;

/*!
 * Invoked once when the traversal is started.
 */
- (void) connectedVerticesTraversalDidStart;

/*!
 * Corresponds to previsit of the vertex.
 */
- (void) willVisitVertex: (vertexType) v;

/*!
 * Invoked on every out-edge of the vertex.
 */
- (void) willTraverseEdge: (edgeType) e;

/*!
 * Invoked when the traversal algorithm *discovers* vertex u.
 */
- (void) foundUndiscoveredVertex: (vertexType) v;

/*!
 * Invoked when the traversal algorithm encounters previously *discovered* vertex.
 */
- (void) foundDiscoveredVertex: (vertexType) v;

/*!
 * Invoked when the traversal algorithm encounters already *processed* vertex.
 */
- (void) foundProcessedVertex: (vertexType) v;

/*!
 * Corresponds to postvisit of the vertex.
 */
- (void) didVisitVertex: (vertexType) v;

/*!
 * Invoked just after the traversal of connected vertices is complete.
 */
- (void) connectedVerticesTraversalDidFinish;

/*!
 * Invoked after traversal of the last connected component if user called whole-graph traversal.
 */
- (void) graphTraversalDidFinish;

@end
