////////////////////////////////////////////////////////////////////////////////
//  
//  UndirectednessAdapter.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 08/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "UndirectednessAdapter.h"

#import "DirectedGraph.h"

@implementation UndirectednessAdapter

@synthesize directedGraph;

+ (UndirectednessAdapter*) adapterWithDirectedGraph: (NSObject<DirectedGraph>*) graph
{
  return [[[[self class] alloc] initWithDirectedGraph: graph] autorelease];
}

- (id) initWithDirectedGraph: (NSObject<DirectedGraph>*) graph
{
  directedGraph = [graph retain];
  
  return self;
}

- (void) dealloc
{
  [directedGraph release], directedGraph = nil;
  
  [super dealloc];
}

- (NSMethodSignature*) methodSignatureForSelector: (SEL) aSelector
{
  return [directedGraph methodSignatureForSelector: aSelector];
}

- (void) forwardInvocation: (NSInvocation*) anInvocation
{
  if([directedGraph respondsToSelector: [anInvocation selector]])
  {
    [anInvocation invokeWithTarget: directedGraph];
  }
  else
  {
    // Вызываем super, чтобы получить NSInvalidArgumentException.
    [super forwardInvocation: anInvocation];
  }
}

#pragma mark - Graph protocol re-implementation

// Возвращаем энумератор всех входящих и исходящих ребер данной вершины. Потом -oppositeVertexOfEdge:currentVertex: делает свое дело.
- (NSEnumerator*) possibleWaysOutEnumeratorOf: (vertexType) vertex
{
  return [[self incidentEdgesSetOf: vertex] objectEnumerator];
}

#pragma mark - UndirectedGraph protocol implementation

- (edgeType) edgeBetween: (vertexType) u and: (vertexType) v
{
  edgeType edge = nil;
  
  if((edge = [directedGraph edgeFrom: u to: v])) return edge;
  
  if((edge = [directedGraph edgeFrom: v to: u])) return edge;
  
  return nil;
}

- (BOOL) containsEdgeBetween: (vertexType) u and: (vertexType) v
{
  return [self edgeBetween: u and: v] != nil;
}

- (Pair*) incidentVerticesPairOf: (edgeType) edge
{
  return [Pair pairWithObject: [directedGraph tailVertexOfEdge: edge] andObject: [directedGraph headVertexOfEdge: edge]];
}

// Warning: порядок вершин в ребрах сохраняется от направленного графа!
- (NSSet*) incidentEdgesSetOf: (vertexType) v
{
  return [[directedGraph incomingEdgesSetOf: v] setByAddingObjectsFromSet: [directedGraph outgoingEdgesSetOf: v]];
}

- (BOOL) isVertex: (vertexType) vertex incidentToEdge: (edgeType) edge
{
  Pair* vPair = [self incidentVerticesPairOf: edge];
  
  return [[vPair object1] isEqual: vertex] || [[vPair object2] isEqual: vertex];
}

- (BOOL) isEdge: (edgeType) edge incidentToVertex: (vertexType) vertex
{
  return [self isVertex: vertex incidentToEdge: edge];
}

- (NSSet*) adjacentEdgesSetOf: (edgeType) edge
{
  NSSet* adjacentE = [self incidentEdgesSetOf: [directedGraph tailVertexOfEdge: edge]];
  
  [adjacentE setByAddingObjectsFromSet: [self incidentEdgesSetOf: [directedGraph headVertexOfEdge: edge]]];
  
  return adjacentE;
}

- (BOOL) isEdge: (edgeType) edge1 adjacentTo: (edgeType) edge2
{
  return [[self adjacentEdgesSetOf: edge1] containsObject: edge2];
}

@end
